"""
iris-ansi terminal info

Revision: 1.1256
Source: https://invisible-mirror.net/archives/ncurses/current/ncurses.tar.gz

This file is derived from the ncurses terminfo database, which is
distributed under the MIT/X11 license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

BOOL_CAPS = [
    'am',    # (auto_right_margin) terminal has automatic margins
]

NUM_CAPS = {
    'cols': 80,     # (columns) number of columns in a line
    'it': 8,        # (init_tabs) tabs initially every # spaces
    'lines': 40,    # (lines) number of lines on screen or page
}

STR_CAPS = {
    'bel': b'\a',
    'bold': b'\x1b[1m',
    'clear': b'\x1b[H\x1b[2J',
    'cnorm': b'\x1b[9/y\x1b[12/y\x1b[=6l',
    'cr': b'\r',
    'cub': b'\x1b[%p1%dD',
    'cub1': b'\x1b[D',
    'cud': b'\x1b[%p1%dB',
    'cud1': b'\n',
    'cuf': b'\x1b[%p1%dC',
    'cuf1': b'\x1b[C',
    'cup': b'\x1b[%i%p1%d;%p2%dH',
    'cuu': b'\x1b[%p1%dA',
    'cuu1': b'\x1b[A',
    'cvvis': b'\x1b[10/y\x1b[=1h\x1b[=2l\x1b[=6h',
    'dl': b'\x1b[%p1%dM',
    'dl1': b'\x1b[M',
    'ed': b'\x1b[J',
    'el': b'\x1b[K',
    'el1': b'\x1b[1K',
    'enacs': b'',
    'home': b'\x1b[H',
    'ht': b'\t',
    'hts': b'\x1bH',
    'il': b'\x1b[%p1%dL',
    'il1': b'\x1b[L',
    'ind': b'\x1bD',
    'is2': b'\x1b[?1l\x1b>\x1b[?7h\x1b[100g\x1b[0m\x1b7\x1b[r\x1b8',
    'kDC': b'\x1b[P',
    'kEND': b'\x1b[147q',
    'kHOM': b'\x1b[143q',
    'kLFT': b'\x1b[158q',
    'kPRT': b'\x1b[210q',
    'kRIT': b'\x1b[167q',
    'kSPD': b'\x1b[218q',
    'kbs': b'\b',
    'kcbt': b'\x1b[Z',
    'kcub1': b'\x1b[D',
    'kcud1': b'\x1b[B',
    'kcuf1': b'\x1b[C',
    'kcuu1': b'\x1b[A',
    'kdch1': b'\x7f',
    'kend': b'\x1b[146q',
    'kent': b'\r',
    'kf1': b'\x1b[001q',
    'kf10': b'\x1b[010q',
    'kf11': b'\x1b[011q',
    'kf12': b'\x1b[012q',
    'kf2': b'\x1b[002q',
    'kf3': b'\x1b[003q',
    'kf4': b'\x1b[004q',
    'kf5': b'\x1b[005q',
    'kf6': b'\x1b[006q',
    'kf7': b'\x1b[007q',
    'kf8': b'\x1b[008q',
    'kf9': b'\x1b[009q',
    'khome': b'\x1b[H',
    'kich1': b'\x1b[139q',
    'knp': b'\x1b[154q',
    'kpp': b'\x1b[150q',
    'kprt': b'\x1b[209q',
    'krmir': b'\x1b[146q',
    'kspd': b'\x1b[217q',
    'nel': b'\x1bE',
    'pfkey': b'\x1bP101;%p1%d.y%p2%s\x1b\x5c',
    'rc': b'\x1b8',
    'rev': b'\x1b[7m',
    'ri': b'\x1bM',
    'rmacs': b'',
    'rmam': b'\x1b[?7l',
    'rmso': b'\x1b[m',
    'rmul': b'\x1b[m',
    's0ds': b'',
    's1ds': b'',
    'sc': b'\x1b7',
    'sgr0': b'\x1b[m',
    'smacs': b'',
    'smam': b'\x1b[?7h',
    'smso': b'\x1b[1;7m',
    'smul': b'\x1b[4m',
    'tbc': b'\x1b[3g',
}



"""
hpterm terminal info

Revision: 1.1256
Source: https://invisible-mirror.net/archives/ncurses/current/ncurses.tar.gz

This file is derived from the ncurses terminfo database, which is
distributed under the MIT/X11 license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

BOOL_CAPS = [
    'am',     # (auto_right_margin) terminal has automatic margins
    'da',     # (memory_above) display may be retained above the screen
    'db',     # (memory_below) display may be retained below the screen
    'mir',    # (move_insert_mode) safe to move while in insert mode
    'xhp',    # (ceol_standout_glitch) standout not erased by overwriting (hp)
    'xon',    # (xon_xoff) terminal uses xon/xoff handshaking
]

NUM_CAPS = {
    'cols': 80,     # (columns) number of columns in a line
    'lh': 2,        # (label_height) rows in each label
    'lines': 24,    # (lines) number of lines on screen or page
    'lm': 0,        # (lines_of_memory) lines of memory if > line. 0 means varies
    'lw': 8,        # (label_width) columns in each label
    'nlab': 8,      # (num_labels) number of labels on screen
    'pb': 9600,     # (padding_baud_rate) lowest baud rate where padding needed
    'xmc': 0,       # (magic_cookie_glitch) number of blank characters left by smso or rmso
}

STR_CAPS = {
    'bel': b'\a',
    'bold': b'\x1b&dB',
    'cbt': b'\x1bi',
    'clear': b'\x1b&a0y0C\x1bJ',
    'cr': b'\r',
    'cub1': b'\b',
    'cud1': b'\x1bB',
    'cuf1': b'\x1bC',
    'cup': b'\x1b&a%p1%dy%p2%dC',
    'cuu1': b'\x1bA',
    'dch1': b'\x1bP',
    'dim': b'\x1b&dH',
    'dl1': b'\x1bM',
    'ed': b'\x1bJ',
    'el': b'\x1bK',
    'enacs': b'',
    'hpa': b'\x1b&a%p1%dC',
    'ht': b'\t',
    'hts': b'\x1b1',
    'il1': b'\x1bL',
    'ind': b'\n',
    'kbs': b'\b',
    'kclr': b'\x1bJ',
    'kctab': b'\x1b2',
    'kcub1': b'\x1bD',
    'kcud1': b'\x1bB',
    'kcuf1': b'\x1bC',
    'kcuu1': b'\x1bA',
    'kdch1': b'\x1bP',
    'kdl1': b'\x1bM',
    'ked': b'\x1bJ',
    'kel': b'\x1bK',
    'kf1': b'\x1bp',
    'kf2': b'\x1bq',
    'kf3': b'\x1br',
    'kf4': b'\x1bs',
    'kf5': b'\x1bt',
    'kf6': b'\x1bu',
    'kf7': b'\x1bv',
    'kf8': b'\x1bw',
    'khome': b'\x1bh',
    'khts': b'\x1b1',
    'kich1': b'\x1bQ',
    'kil1': b'\x1bL',
    'kind': b'\x1bS',
    'kll': b'\x1bF',
    'knp': b'\x1bU',
    'kpp': b'\x1bV',
    'kri': b'\x1bT',
    'krmir': b'\x1bR',
    'ktbc': b'\x1b3',
    'meml': b'\x1bl',
    'memu': b'\x1bm',
    'pfkey': b'\x1b&f%p1%dk%p2%l%dL%p2%s',
    'pfloc': b'\x1b&f1a%p1%dk%p2%l%dL%p2%s',
    'pfx': b'\x1b&f2a%p1%dk%p2%l%dL%p2%s',
    'pln': b'\x1b&f%p1%dk%p2%l%dd0L%p2%s',
    'rev': b'\x1b&dB',
    'ri': b'\x1bT',
    'rmacs': b'',
    'rmir': b'\x1bR',
    'rmkx': b'\x1b&s0A',
    'rmln': b'\x1b&j@',
    'rmso': b'\x1b&d@',
    'rmul': b'\x1b&d@',
    's0ds': b'',
    's1ds': b'',
    'sgr': b'\x1b&d%?%p7%t%\x27s\x27%c%;%p1%p3%|%p6%|%{2}%*%p2%{4}%*%+%p4%+%p5%{8}%*%+%\x27@\x27%+%c%?%p9%t%\x27\x27%c%e%\x27\x27%c%;',
    'sgr0': b'\x1b&d@',
    'smacs': b'',
    'smir': b'\x1bQ',
    'smkx': b'\x1b&s1A',
    'smln': b'\x1b&jB',
    'smso': b'\x1b&dJ',
    'smul': b'\x1b&dD',
    'tbc': b'\x1b3',
    'vpa': b'\x1b&a%p1%dY',
}



"""
dtterm terminal info

Revision: 1.1256
Source: https://invisible-mirror.net/archives/ncurses/current/ncurses.tar.gz

This file is derived from the ncurses terminfo database, which is
distributed under the MIT/X11 license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

BOOL_CAPS = [
    'am',      # (auto_right_margin) terminal has automatic margins
    'mir',     # (move_insert_mode) safe to move while in insert mode
    'msgr',    # (move_standout_mode) safe to move while in standout mode
    'xenl',    # (eat_newline_glitch) newline ignored after 80 cols (concept)
    'xon',     # (xon_xoff) terminal uses xon/xoff handshaking
    'AX',      # (ansi_x3.64_1979) terminal uses ECMA-48/ANSI X3.64 color sequences
]

NUM_CAPS = {
    'colors': 8,    # (max_colors) maximum number of colors on screen
    'cols': 80,     # (columns) number of columns in a line
    'it': 8,        # (init_tabs) tabs initially every # spaces
    'lines': 24,    # (lines) number of lines on screen or page
    'lm': 0,        # (lines_of_memory) lines of memory if > line. 0 means varies
    'pairs': 64,    # (max_pairs) maximum number of color-pairs on the screen
}

STR_CAPS = {
    'acsc': b'``aaffggjjkkllmmnnooppqqrrssttuuvvwwxxyyzz{{||}}~~',
    'bel': b'\a',
    'blink': b'\x1b[5m',
    'bold': b'\x1b[1m',
    'civis': b'\x1b[?25l',
    'clear': b'\x1b[H\x1b[J',
    'cnorm': b'\x1b[?25h',
    'cr': b'\r',
    'csr': b'\x1b[%i%p1%d;%p2%dr',
    'cub': b'\x1b[%p1%dD',
    'cub1': b'\b',
    'cud': b'\x1b[%p1%dB',
    'cud1': b'\n',
    'cuf': b'\x1b[%p1%dC',
    'cuf1': b'\x1b[C',
    'cup': b'\x1b[%i%p1%d;%p2%dH',
    'cuu': b'\x1b[%p1%dA',
    'cuu1': b'\x1b[A',
    'dch': b'\x1b[%p1%dP',
    'dch1': b'\x1b[P',
    'dim': b'\x1b[2m',
    'dl': b'\x1b[%p1%dM',
    'dl1': b'\x1b[M',
    'ech': b'\x1b[%p1%dX',
    'ed': b'\x1b[J',
    'el': b'\x1b[K',
    'el1': b'\x1b[1K',
    'enacs': b'',
    'flash': b'\x1b[?5h\x1b[?5l',
    'home': b'\x1b[H',
    'ht': b'\t',
    'hts': b'\x1bH',
    'ich': b'\x1b[%p1%d@',
    'il': b'\x1b[%p1%dL',
    'il1': b'\x1b[L',
    'ind': b'\x1bD',
    'invis': b'\x1b[8m',
    'is2': b'\x1b F\x1b>\x1b[?1l\x1b[?7h\x1b[?45l',
    'kbs': b'\b',
    'kcub1': b'\x1b[D',
    'kcud1': b'\x1b[B',
    'kcuf1': b'\x1b[C',
    'kcuu1': b'\x1b[A',
    'kdch1': b'\x1b[3~',
    'kf1': b'\x1b[11~',
    'kf10': b'\x1b[21~',
    'kf11': b'\x1b[23~',
    'kf12': b'\x1b[24~',
    'kf13': b'\x1b[25~',
    'kf14': b'\x1b[26~',
    'kf15': b'\x1b[28~',
    'kf16': b'\x1b[29~',
    'kf17': b'\x1b[31~',
    'kf18': b'\x1b[32~',
    'kf19': b'\x1b[33~',
    'kf2': b'\x1b[12~',
    'kf20': b'\x1b[34~',
    'kf3': b'\x1b[13~',
    'kf4': b'\x1b[14~',
    'kf5': b'\x1b[15~',
    'kf6': b'\x1b[17~',
    'kf7': b'\x1b[18~',
    'kf8': b'\x1b[19~',
    'kf9': b'\x1b[20~',
    'kfnd': b'\x1b[1~',
    'khlp': b'\x1b[28~',
    'kich1': b'\x1b[2~',
    'knp': b'\x1b[6~',
    'kpp': b'\x1b[5~',
    'kslt': b'\x1b[4~',
    'nel': b'\x1bE',
    'op': b'\x1b[39;49m',
    'rc': b'\x1b8',
    'rev': b'\x1b[7m',
    'ri': b'\x1bM',
    'rmacs': b'',
    'rmam': b'\x1b[?7l',
    'rmir': b'\x1b[4l',
    'rmso': b'\x1b[22;27m',
    'rmul': b'\x1b[24m',
    's0ds': b'',
    's1ds': b'',
    'sc': b'\x1b7',
    'setab': b'\x1b[4%p1%dm',
    'setaf': b'\x1b[3%p1%dm',
    'sgr': b'\x1b[0%?%p1%t;2;7%;%?%p2%t;4%;%?%p3%t;7%;%?%p4%t;5%;%?%p5%t;2%;%?%p6%t;1%;%?%p7%t;8%;m',
    'sgr0': b'\x1b[m',
    'smacs': b'',
    'smam': b'\x1b[?7h',
    'smir': b'\x1b[4h',
    'smso': b'\x1b[2;7m',
    'smul': b'\x1b[4m',
    'tbc': b'\x1b[3g',
    'u6': b'\x1b[%i%d;%dR',
    'u7': b'\x1b[6n',
    'u8': b'\x1b[?%[;0123456789]c',
    'u9': b'\x1bZ',
}



"""
rio terminal info (derived from alacritty)

Revision: 1.1256
Source: https://invisible-mirror.net/archives/ncurses/current/ncurses.tar.gz

This file is derived from the ncurses terminfo database, which is
distributed under the MIT/X11 license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

from .alacritty import BOOL_CAPS, NUM_CAPS, STR_CAPS

BOOL_CAPS = BOOL_CAPS[:]
NUM_CAPS = NUM_CAPS.copy()
STR_CAPS = STR_CAPS.copy()



"""
Ansicon virtual terminal codes

Information sourced from:
    https://github.com/adoxa/ansicon/blob/master/sequences.txt

A best effort has been made, but not all information was available
"""

from .xterm_256color import BOOL_CAPS, NUM_CAPS, STR_CAPS

BOOL_CAPS = BOOL_CAPS[:]
NUM_CAPS = NUM_CAPS.copy()
STR_CAPS = STR_CAPS.copy()
STR_CAPS['enacs'] = b''  # empty (G0/SO/SI stripped)
STR_CAPS['rmacs'] = b''  # empty (G0/SO/SI stripped)
STR_CAPS['s0ds'] = b''  # empty (G0/SO/SI stripped)
STR_CAPS['s1ds'] = b''  # empty (G0/SO/SI stripped)
STR_CAPS['smacs'] = b''  # empty (G0/SO/SI stripped)


# Added
STR_CAPS['cht'] = b'\x1b[%p1%dI'
STR_CAPS['cnl'] = b'\x1b[%p1%dE'
STR_CAPS['cpl'] = b'\x1b[%p1%dF'
STR_CAPS['da1'] = b'\x1b[0c'
STR_CAPS['dsr'] = b'\x1b[5n'
STR_CAPS['hvp'] = b'\x1b[%i%p1%d;%p2%df'  # Same as cup
STR_CAPS['setb'] = b'\x1b[48;5;%p1%dm'
STR_CAPS['setf'] = b'\x1b[38;5;%p1%dm'

# Removed - These do not appear to be supported
for _drop in ('dim', 'flash', 'invis', 'kcbt', 'kEND', 'kf37', 'kf38', 'kf39',
              'kf40', 'kf41', 'kf42', 'kf43', 'kf44', 'kf45', 'kf46', 'kf47',
              'kf48', 'kf61', 'kf62', 'kf63', 'kIC', 'kind', 'kLFT', 'kmous',
              'kNXT', 'kPRV', 'kri', 'kRIT', 'meml', 'memu', 'ritm', 'rmam',
              'rmcup', 'rmir', 'rmkx', 'rmm', 'sitm', 'smam', 'smcup', 'smir',
              'smkx', 'smm'):
    STR_CAPS.pop(_drop, None)

# Modified
NUM_CAPS['colors'] = 16
NUM_CAPS['cols'] = 80
NUM_CAPS['lines'] = 30
NUM_CAPS['pairs'] = 256
STR_CAPS['cbt'] = b'\x1b[%p1%dZ'
STR_CAPS['cnorm'] = b'\x1b[?25h'
STR_CAPS['csr'] = b'\x1b[%p1%{1}%+%d;%?%p2%t%p2%{1}%+%dr'
STR_CAPS['cub1'] = b'\x1b[D'
STR_CAPS['cud1'] = b'\x1b[B'
STR_CAPS['cvvis'] = b'\x1b[?25h'
STR_CAPS['initc'] = b'\x1b]4;%p1%d;rgb] =%p2%d/%p3%d/%p4%d\x1b'
STR_CAPS['is2'] = b'\x1b[!p\x1b>'
STR_CAPS['ka1'] = b'\x00G'   # upper left of keypad
STR_CAPS['ka3'] = b'\x00I'   # lower right of keypad
STR_CAPS['kbs'] = b'\x08'
STR_CAPS['kc1'] = b'\x00O'   # lower left of keypad
STR_CAPS['kc3'] = b'\x00Q'   # lower right of keypad
STR_CAPS['kcub1'] = b'\xe0K'
STR_CAPS['kcud1'] = b'\xe0P'
STR_CAPS['kcuf1'] = b'\xe0M'
STR_CAPS['kcuu1'] = b'\xe0H'
STR_CAPS['kDC'] = b'\xe0S'
STR_CAPS['kdch1'] = b'\x0eQ'
STR_CAPS['kend'] = b'\xe0O'
STR_CAPS['kent'] = b'\r'
STR_CAPS['kf1'] = b'\x00;'
STR_CAPS['kf2'] = b'\x00<'
STR_CAPS['kf3'] = b'\x00='
STR_CAPS['kf4'] = b'\x00>'
STR_CAPS['kf5'] = b'\x00?'
STR_CAPS['kf6'] = b'\x00@'
STR_CAPS['kf7'] = b'\x00A'
STR_CAPS['kf8'] = b'\x00B'
STR_CAPS['kf9'] = b'\x00C'
STR_CAPS['kf10'] = b'\x00D'
STR_CAPS['kf11'] = b'\xe0\x85'
STR_CAPS['kf12'] = b'\xe0\x86'
STR_CAPS['kf13'] = b'\x00T'
STR_CAPS['kf14'] = b'\x00U'
STR_CAPS['kf15'] = b'\x00V'
STR_CAPS['kf16'] = b'\x00W'
STR_CAPS['kf17'] = b'\x00X'
STR_CAPS['kf18'] = b'\x00Y'
STR_CAPS['kf19'] = b'\x00Z'
STR_CAPS['kf20'] = b'\x00['
STR_CAPS['kf21'] = b'\x00\\'
STR_CAPS['kf22'] = b'\x00]'
STR_CAPS['kf23'] = b'\xe0\x87'
STR_CAPS['kf24'] = b'\xe0\x88'
STR_CAPS['kf25'] = b'\x00^'
STR_CAPS['kf26'] = b'\x00_'
STR_CAPS['kf27'] = b'\x00`'
STR_CAPS['kf28'] = b'\x00a'
STR_CAPS['kf29'] = b'\x00b'
STR_CAPS['kf30'] = b'\x00c'
STR_CAPS['kf31'] = b'\x00d'
STR_CAPS['kf32'] = b'\x00e'
STR_CAPS['kf33'] = b'\x00f'
STR_CAPS['kf34'] = b'\x00g'
STR_CAPS['kf35'] = b'\xe0\x89'
STR_CAPS['kf36'] = b'\xe0\x8a'
# Missing F37 - F48
STR_CAPS['kf49'] = b'\x00h'
STR_CAPS['kf50'] = b'\x00i'
STR_CAPS['kf51'] = b'\x00j'
STR_CAPS['kf52'] = b'\x00k'
STR_CAPS['kf53'] = b'\x00l'
STR_CAPS['kf54'] = b'\x00m'
STR_CAPS['kf55'] = b'\x00n'
STR_CAPS['kf56'] = b'\x00o'
STR_CAPS['kf57'] = b'\x00p'
STR_CAPS['kf58'] = b'\x00q'
STR_CAPS['kf59'] = b'\xe0\x8b'
STR_CAPS['kf60'] = b'\xe0\x8b'
# Missing F61 - F63
STR_CAPS['khome'] = b'\xe0G'
STR_CAPS['kich1'] = b'\xe0R'
STR_CAPS['knp'] = b'\xe0Q'
STR_CAPS['kpp'] = b'\xe0I'
STR_CAPS['rs1'] = b'\x1bc\x1b]104ST'
STR_CAPS['rs2'] = b'\x1b[!p'
STR_CAPS['sgr'] = b'\x1b[%p1%d%?%p2%t;%p2%d%;%?%p3%t;%p3%d%;%?%p4%t;%p4%d%;%?%p5%t;%p5%d%;' \
                  b'%?%p6%t;%p6%d%;%?%p7%t;%p7%d%;%?%p8%t;%p8%d%;%?%p9%t;%p9%d%;m'



"""
sun terminal info

Revision: 1.1256
Source: https://invisible-mirror.net/archives/ncurses/current/ncurses.tar.gz

This file is derived from the ncurses terminfo database, which is
distributed under the MIT/X11 license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

BOOL_CAPS = [
    'am',      # (auto_right_margin) terminal has automatic margins
    'km',      # (has_meta_key) Has a meta key (i.e., sets 8th-bit)
    'msgr',    # (move_standout_mode) safe to move while in standout mode
]

NUM_CAPS = {
    'cols': 80,     # (columns) number of columns in a line
    'lines': 34,    # (lines) number of lines on screen or page
}

STR_CAPS = {
    'bel': b'\a',
    'clear': b'\f',
    'cr': b'\r',
    'cub1': b'\b',
    'cud1': b'\n',
    'cuf1': b'\x1b[C',
    'cup': b'\x1b[%i%p1%d;%p2%dH',
    'cuu1': b'\x1b[A',
    'dch': b'\x1b[%p1%dP',
    'dch1': b'\x1b[P',
    'dl': b'\x1b[%p1%dM',
    'dl1': b'\x1b[M',
    'ed': b'\x1b[J',
    'el': b'\x1b[K',
    'enacs': b'',
    'ht': b'\t',
    'ich': b'\x1b[%p1%d@',
    'ich1': b'\x1b[@',
    'il': b'\x1b[%p1%dL',
    'il1': b'\x1b[L',
    'ind': b'\n',
    'kb2': b'\x1b[218z',
    'kbs': b'\b',
    'kcub1': b'\x1b[D',
    'kcud1': b'\x1b[B',
    'kcuf1': b'\x1b[C',
    'kcuu1': b'\x1b[A',
    'kdch1': b'\x7f',
    'kend': b'\x1b[220z',
    'kf1': b'\x1b[224z',
    'kf10': b'\x1b[233z',
    'kf11': b'\x1b[234z',
    'kf12': b'\x1b[235z',
    'kf2': b'\x1b[225z',
    'kf3': b'\x1b[226z',
    'kf4': b'\x1b[227z',
    'kf5': b'\x1b[228z',
    'kf6': b'\x1b[229z',
    'kf7': b'\x1b[230z',
    'kf8': b'\x1b[231z',
    'kf9': b'\x1b[232z',
    'khome': b'\x1b[214z',
    'kich1': b'\x1b[247z',
    'knp': b'\x1b[222z',
    'kopt': b'\x1b[194z',
    'kpp': b'\x1b[216z',
    'kres': b'\x1b[193z',
    'kund': b'\x1b[195z',
    'rev': b'\x1b[7m',
    'rmacs': b'',
    'rmso': b'\x1b[m',
    'rs2': b'\x1b[s',
    's0ds': b'',
    's1ds': b'',
    'sgr': b'\x1b[0%?%p1%p3%|%t;7%;m',
    'sgr0': b'\x1b[m',
    'smacs': b'',
    'smso': b'\x1b[7m',
    'u8': b'\x1b[1t',
    'u9': b'\x1b[11t',
}



"""
Key code constants

Most of this information came from the terminfo man pages, part of ncurses
More information on ncurses can be found at:
https://www.gnu.org/software/ncurses/ncurses.html
"""

KEY_A1 = 348
KEY_A3 = 349
KEY_B2 = 350
KEY_BACKSPACE = 263
KEY_BEG = 354
KEY_BREAK = 257
KEY_BTAB = 353
KEY_C1 = 351
KEY_C3 = 352
KEY_CANCEL = 355
KEY_CATAB = 342
KEY_CLEAR = 333
KEY_CLOSE = 356
KEY_COMMAND = 357
KEY_COPY = 358
KEY_CREATE = 359
KEY_CTAB = 341
KEY_DC = 330
KEY_DL = 328
KEY_DOWN = 258
KEY_EIC = 332
KEY_END = 360
KEY_ENTER = 343
KEY_EOL = 335
KEY_EOS = 334
KEY_EXIT = 361
KEY_F0 = 264
KEY_F1 = 265
KEY_F10 = 274
KEY_F11 = 275
KEY_F12 = 276
KEY_F13 = 277
KEY_F14 = 278
KEY_F15 = 279
KEY_F16 = 280
KEY_F17 = 281
KEY_F18 = 282
KEY_F19 = 283
KEY_F2 = 266
KEY_F20 = 284
KEY_F21 = 285
KEY_F22 = 286
KEY_F23 = 287
KEY_F24 = 288
KEY_F25 = 289
KEY_F26 = 290
KEY_F27 = 291
KEY_F28 = 292
KEY_F29 = 293
KEY_F3 = 267
KEY_F30 = 294
KEY_F31 = 295
KEY_F32 = 296
KEY_F33 = 297
KEY_F34 = 298
KEY_F35 = 299
KEY_F36 = 300
KEY_F37 = 301
KEY_F38 = 302
KEY_F39 = 303
KEY_F4 = 268
KEY_F40 = 304
KEY_F41 = 305
KEY_F42 = 306
KEY_F43 = 307
KEY_F44 = 308
KEY_F45 = 309
KEY_F46 = 310
KEY_F47 = 311
KEY_F48 = 312
KEY_F49 = 313
KEY_F5 = 269
KEY_F50 = 314
KEY_F51 = 315
KEY_F52 = 316
KEY_F53 = 317
KEY_F54 = 318
KEY_F55 = 319
KEY_F56 = 320
KEY_F57 = 321
KEY_F58 = 322
KEY_F59 = 323
KEY_F6 = 270
KEY_F60 = 324
KEY_F61 = 325
KEY_F62 = 326
KEY_F63 = 327
KEY_F7 = 271
KEY_F8 = 272
KEY_F9 = 273
KEY_FIND = 362
KEY_HELP = 363
KEY_HOME = 262
KEY_IC = 331
KEY_IL = 329
KEY_LEFT = 260
KEY_LL = 347
KEY_MARK = 364
KEY_MAX = 511
KEY_MESSAGE = 365
KEY_MIN = 257
KEY_MOUSE = 409
KEY_MOVE = 366
KEY_NEXT = 367
KEY_NPAGE = 338
KEY_OPEN = 368
KEY_OPTIONS = 369
KEY_PPAGE = 339
KEY_PREVIOUS = 370
KEY_PRINT = 346
KEY_REDO = 371
KEY_REFERENCE = 372
KEY_REFRESH = 373
KEY_REPLACE = 374
KEY_RESET = 345
KEY_RESIZE = 410
KEY_RESTART = 375
KEY_RESUME = 376
KEY_RIGHT = 261
KEY_SAVE = 377
KEY_SBEG = 378
KEY_SCANCEL = 379
KEY_SCOMMAND = 380
KEY_SCOPY = 381
KEY_SCREATE = 382
KEY_SDC = 383
KEY_SDL = 384
KEY_SELECT = 385
KEY_SEND = 386
KEY_SEOL = 387
KEY_SEXIT = 388
KEY_SF = 336
KEY_SFIND = 389
KEY_SHELP = 390
KEY_SHOME = 391
KEY_SIC = 392
KEY_SLEFT = 393
KEY_SMESSAGE = 394
KEY_SMOVE = 395
KEY_SNEXT = 396
KEY_SOPTIONS = 397
KEY_SPREVIOUS = 398
KEY_SPRINT = 399
KEY_SR = 337
KEY_SREDO = 400
KEY_SREPLACE = 401
KEY_SRESET = 344
KEY_SRIGHT = 402
KEY_SRSUME = 403
KEY_SSAVE = 404
KEY_SSUSPEND = 405
KEY_STAB = 340
KEY_SUNDO = 406
KEY_SUSPEND = 407
KEY_UNDO = 408
KEY_UP = 259



# -*- coding: utf-8 -*-
# Copyright 2019 - 2026 Avram Lubkin, All Rights Reserved

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# pylint: disable=too-few-public-methods,invalid-name
"""
Support functions and wrappers for calls to the Windows API
"""

import atexit
import codecs
from collections import namedtuple
import ctypes
from ctypes import wintypes
import io
import msvcrt  # pylint: disable=import-error
import os
import platform
import sys

from jinxed._util import mock, IS_WINDOWS

# Workaround for auto-doc generation on Linux
if not IS_WINDOWS:
    ctypes = mock.Mock()  # noqa: F811

LPDWORD = ctypes.POINTER(wintypes.DWORD)
COORD = wintypes._COORD  # pylint: disable=protected-access

# Console input modes
ENABLE_ECHO_INPUT = 0x0004
ENABLE_EXTENDED_FLAGS = 0x0080
ENABLE_INSERT_MODE = 0x0020
ENABLE_LINE_INPUT = 0x0002
ENABLE_MOUSE_INPUT = 0x0010
ENABLE_PROCESSED_INPUT = 0x0001
ENABLE_QUICK_EDIT_MODE = 0x0040
ENABLE_WINDOW_INPUT = 0x0008
ENABLE_VIRTUAL_TERMINAL_INPUT = 0x0200

# Console output modes
ENABLE_PROCESSED_OUTPUT = 0x0001
ENABLE_WRAP_AT_EOL_OUTPUT = 0x0002
ENABLE_VIRTUAL_TERMINAL_PROCESSING = 0x0004
DISABLE_NEWLINE_AUTO_RETURN = 0x0008
ENABLE_LVB_GRID_WORLDWIDE = 0x0010

if IS_WINDOWS and tuple(int(num) for num in platform.version().split('.')) >= (10, 0, 10586):
    VTMODE_SUPPORTED = True
    CBREAK_MODE = ENABLE_PROCESSED_INPUT | ENABLE_VIRTUAL_TERMINAL_INPUT
    RAW_MODE = ENABLE_VIRTUAL_TERMINAL_INPUT
else:
    VTMODE_SUPPORTED = False
    CBREAK_MODE = ENABLE_PROCESSED_INPUT
    RAW_MODE = 0

GTS_SUPPORTED = hasattr(os, 'get_terminal_size')
TerminalSize = namedtuple('TerminalSize', ('columns', 'lines'))


class ConsoleScreenBufferInfo(ctypes.Structure):
    """
    Python representation of CONSOLE_SCREEN_BUFFER_INFO structure
    https://learn.microsoft.com/en-us/windows/console/console-screen-buffer-info-str
    """

    _fields_ = [('dwSize', COORD),
                ('dwCursorPosition', COORD),
                ('wAttributes', wintypes.WORD),
                ('srWindow', wintypes.SMALL_RECT),
                ('dwMaximumWindowSize', COORD)]


CSBIP = ctypes.POINTER(ConsoleScreenBufferInfo)


def _check_bool(result, func, args):  # pylint: disable=unused-argument
    """
    Used as an error handler for Windows calls
    Gets last error if call is not successful
    """

    if not result:
        raise ctypes.WinError(ctypes.get_last_error())
    return args


KERNEL32 = ctypes.WinDLL('kernel32', use_last_error=True)

KERNEL32.GetConsoleCP.errcheck = _check_bool
KERNEL32.GetConsoleCP.argtypes = ()

KERNEL32.GetConsoleMode.errcheck = _check_bool
KERNEL32.GetConsoleMode.argtypes = (wintypes.HANDLE, LPDWORD)

KERNEL32.SetConsoleMode.errcheck = _check_bool
KERNEL32.SetConsoleMode.argtypes = (wintypes.HANDLE, wintypes.DWORD)

KERNEL32.GetConsoleScreenBufferInfo.errcheck = _check_bool
KERNEL32.GetConsoleScreenBufferInfo.argtypes = (wintypes.HANDLE, CSBIP)

KERNEL32.WaitForSingleObject.argtypes = (wintypes.HANDLE, wintypes.DWORD)
KERNEL32.WaitForSingleObject.restype = wintypes.DWORD

# Console input record structures for PeekConsoleInputW/ReadConsoleInputW.


class _KEY_EVENT_RECORD(ctypes.Structure):
    """KEY_EVENT_RECORD structure."""

    _fields_ = [
        ('bKeyDown', wintypes.BOOL),
        ('wRepeatCount', wintypes.WORD),
        ('wVirtualKeyCode', wintypes.WORD),
        ('wVirtualScanCode', wintypes.WORD),
        ('uChar', wintypes.WCHAR),
        ('dwControlKeyState', wintypes.DWORD),
    ]


class _MOUSE_EVENT_RECORD(ctypes.Structure):
    """MOUSE_EVENT_RECORD structure."""

    _fields_ = [
        ('dwMousePosition', COORD),
        ('dwButtonState', wintypes.DWORD),
        ('dwControlKeyState', wintypes.DWORD),
        ('dwEventFlags', wintypes.DWORD),
    ]


class _WINDOW_BUFFER_SIZE_RECORD(ctypes.Structure):
    """WINDOW_BUFFER_SIZE_RECORD structure."""

    _fields_ = [('dwSize', COORD)]


class _INPUT_RECORD_EVENT(ctypes.Union):
    """Union of console input event types."""

    _fields_ = [
        ('KeyEvent', _KEY_EVENT_RECORD),
        ('MouseEvent', _MOUSE_EVENT_RECORD),
        ('WindowBufferSizeEvent', _WINDOW_BUFFER_SIZE_RECORD),
    ]


class INPUT_RECORD(ctypes.Structure):
    """INPUT_RECORD structure.

    .. py:attribute:: EventType

        Event type constant:

        - ``0x0001`` -- KEY_EVENT
        - ``0x0002`` -- MOUSE_EVENT
        - ``0x0004`` -- WINDOW_BUFFER_SIZE_EVENT

    .. py:attribute:: Event

        Union of event data.  Access the member matching ``EventType``:

        - ``Event.KeyEvent`` -- ``bKeyDown``, ``wRepeatCount``,
          ``wVirtualKeyCode``, ``wVirtualScanCode``, ``uChar``,
          ``dwControlKeyState``
        - ``Event.MouseEvent`` -- ``dwMousePosition``, ``dwButtonState``,
          ``dwControlKeyState``, ``dwEventFlags``
        - ``Event.WindowBufferSizeEvent`` -- ``dwSize``
    """

    _fields_ = [
        ('EventType', wintypes.WORD),
        ('Event', _INPUT_RECORD_EVENT),
    ]


KEY_EVENT = 0x0001
MOUSE_EVENT = 0x0002
WINDOW_BUFFER_SIZE_EVENT = 0x0004

KERNEL32.PeekConsoleInputW.argtypes = (
    wintypes.HANDLE, ctypes.POINTER(INPUT_RECORD), wintypes.DWORD, LPDWORD)
KERNEL32.PeekConsoleInputW.restype = wintypes.BOOL

KERNEL32.ReadConsoleInputW.argtypes = (
    wintypes.HANDLE, ctypes.POINTER(INPUT_RECORD), wintypes.DWORD, LPDWORD)
KERNEL32.ReadConsoleInputW.restype = wintypes.BOOL


def get_csbi(filehandle=None):
    """
    Args:
        filehandle(int): Windows filehandle object as returned by :py:func:`msvcrt.get_osfhandle`

    Returns:
        :py:class:`ConsoleScreenBufferInfo`: CONSOLE_SCREEN_BUFFER_INFO_ structure

    Wrapper for GetConsoleScreenBufferInfo_

    If ``filehandle`` is :py:data:`None`, uses the filehandle of :py:data:`sys.__stdout__`.

    """

    if filehandle is None:
        filehandle = msvcrt.get_osfhandle(sys.__stdout__.fileno())

    csbi = ConsoleScreenBufferInfo()
    KERNEL32.GetConsoleScreenBufferInfo(filehandle, ctypes.byref(csbi))
    return csbi


def get_console_input_encoding():
    """
    Returns:
        int: Current console mode

    Query for the console input code page and provide an encoding

    If the code page can not be resolved to a Python encoding, :py:data:`None` is returned.
    """

    try:
        encoding = 'cp%d' % KERNEL32.GetConsoleCP()
    except OSError:
        return None

    try:
        codecs.lookup(encoding)
    except LookupError:
        return None

    return encoding


def get_console_mode(filehandle):
    """
    Args:
        filehandle(int): Windows filehandle object as returned by :py:func:`msvcrt.get_osfhandle`

    Returns:
        int: Current console mode

    Raises:
        OSError: Error calling Windows API

    Wrapper for GetConsoleMode_
    """

    mode = wintypes.DWORD()
    KERNEL32.GetConsoleMode(filehandle, ctypes.byref(mode))
    return mode.value


def set_console_mode(filehandle, mode):
    """
    Args:
        filehandle(int): Windows filehandle object as returned by :py:func:`msvcrt.get_osfhandle`
        mode(int): Desired console mode

    Raises:
        OSError: Error calling Windows API

    Wrapper for SetConsoleMode_
    """

    return bool(KERNEL32.SetConsoleMode(filehandle, mode))


def setcbreak(filehandle):
    """
    Args:
        filehandle(int): Windows filehandle object as returned by :py:func:`msvcrt.get_osfhandle`

    Raises:
        OSError: Error calling Windows API

    Convenience function which mimics :py:func:`tty.setcbreak` behavior

    All console input options are disabled except ``ENABLE_PROCESSED_INPUT``
    and, if supported, ``ENABLE_VIRTUAL_TERMINAL_INPUT``
    """

    set_console_mode(filehandle, CBREAK_MODE)


def setraw(filehandle):
    """
    Args:
        filehandle(int): Windows filehandle object as returned by :py:func:`msvcrt.get_osfhandle`

    Raises:
        OSError: Error calling Windows API

    Convenience function which mimics :py:func:`tty.setraw` behavior

    All console input options are disabled except, if supported, ``ENABLE_VIRTUAL_TERMINAL_INPUT``
    """

    set_console_mode(filehandle, RAW_MODE)


def enable_vt_mode(filehandle=None):
    """
    Args:
        filehandle(int): Windows filehandle object as returned by :py:func:`msvcrt.get_osfhandle`

    Raises:
        OSError: Error calling Windows API

    Enables virtual terminal processing mode for the given console

    If ``filehandle`` is :py:data:`None`, uses the filehandle of :py:data:`sys.__stdout__`.
    """

    if filehandle is None:
        filehandle = msvcrt.get_osfhandle(sys.__stdout__.fileno())

    mode = get_console_mode(filehandle)
    mode |= ENABLE_VIRTUAL_TERMINAL_PROCESSING
    set_console_mode(filehandle, mode)


def get_terminal_size(fd):  # pylint:  disable=invalid-name
    """
    Args:
        fd(int): Python file descriptor

    Returns:
        :py:class:`os.terminal_size`: Named tuple representing terminal size

    Convenience function for getting terminal size

    In Python 3.3 and above, this is a wrapper for :py:func:`os.get_terminal_size`.
    In older versions of Python, this function calls GetConsoleScreenBufferInfo_.
    """

    # In Python 3.3+ we can let the standard library handle this
    if GTS_SUPPORTED:
        return os.get_terminal_size(fd)

    handle = msvcrt.get_osfhandle(fd)
    window = get_csbi(handle).srWindow
    return TerminalSize(window.Right - window.Left + 1, window.Bottom - window.Top + 1)


def flush_and_set_console(fd, mode=None):  # pylint:  disable=invalid-name
    """
    Args:
        filehandle(int): Windows filehandle object as returned by :py:func:`msvcrt.get_osfhandle`
        mode(int): Desired console mode. If None, mode is not changed

    Attempts to set console to specified mode, but will not raise on failure

    If the file descriptor is STDOUT or STDERR, attempts to flush first
    """

    try:
        if fd in (sys.__stdout__.fileno(), sys.__stderr__.fileno()):
            sys.__stdout__.flush()
            sys.__stderr__.flush()
    except (AttributeError, TypeError, io.UnsupportedOperation):
        pass

    if mode is not None:

        try:
            filehandle = msvcrt.get_osfhandle(fd)
            set_console_mode(filehandle, mode)
        except OSError:
            pass


def get_term(fd, fallback=True):  # pylint:  disable=invalid-name
    """
    Args:
        fd(int): Python file descriptor
        fallback(bool): Use fallback terminal type if type can not be determined
    Returns:
        str: Terminal type

    Attempts to determine and enable the current terminal type

    The current logic is:

        - If TERM is defined in the environment, the value is returned
        - Else, if ANSICON is defined in the environment, ``'ansicon'`` is returned
        - Else, if ``WT_SESSION`` is set (Windows Terminal, a modern host
          application distinct from the legacy console host),
          ``'ms-terminal'`` is returned
        - Else, if virtual terminal mode is natively supported
          (legacy Windows Console Host, conhost.exe),
          it is enabled and ``'vtwin10'`` is returned
        - Else, if ``fallback`` is ``True``, Ansicon is loaded, and ``'ansicon'`` is returned
        - If no other conditions are satisfied, ``'unknown'`` is returned

    This logic may change in the future as additional terminal types are added.
    """

    # First try TERM
    term = os.environ.get('TERM', None)

    if term is None:

        # See if ansicon is enabled
        if os.environ.get('ANSICON', None):
            term = 'ansicon'

        # See if Windows Terminal is being used Windows Terminal (wt.exe, now Terminal.exe) is a
        # modern host application distinct from the legacy Windows Console Host (conhost.exe).
        # On GitHub as https://github.com/microsoft/terminal
        elif os.environ.get('WT_SESSION', None):
            term = 'ms-terminal'

        # See if the version of Windows supports VTMODE
        # The legacy Windows Console Host (conhost.exe) supports a subset of
        # VT sequences; its terminfo is hand-maintained as "vtwin10":
        #   https://learn.microsoft.com/en-us/windows/console/console-virtual-terminal-sequences
        elif VTMODE_SUPPORTED:
            try:
                filehandle = msvcrt.get_osfhandle(fd)
                mode = get_console_mode(filehandle)
            except OSError:
                term = 'unknown'
            else:
                # Proceed if VT mode is already enabled
                if mode & ENABLE_VIRTUAL_TERMINAL_PROCESSING:
                    atexit.register(flush_and_set_console, fd, None)

                # Otherwise, enable VT mode
                else:
                    atexit.register(flush_and_set_console, fd, mode)
                    # pylint: disable=unsupported-binary-operation
                    set_console_mode(filehandle, mode | ENABLE_VIRTUAL_TERMINAL_PROCESSING)

                term = 'vtwin10'

        # Currently falling back to Ansicon for older versions of Windows
        elif fallback:
            import ansicon  # pylint: disable=import-error,import-outside-toplevel
            ansicon.load()

            try:
                filehandle = msvcrt.get_osfhandle(fd)
                mode = get_console_mode(filehandle)
            except OSError:
                term = 'unknown'
            else:
                atexit.register(flush_and_set_console, fd, mode)
                set_console_mode(filehandle, mode ^ ENABLE_WRAP_AT_EOL_OUTPUT)
                term = 'ansicon'

        else:
            term = 'unknown'

    return term


class ConsoleInput:
    """Console input handle with wait, peek, and read operations.

    :arg int fd: Python file descriptor (e.g. ``sys.stdin.fileno()``).

    Example::

        console = ConsoleInput(sys.stdin.fileno())
        if console.wait(timeout=5.0):
            event = console.peek()
    """

    def __init__(self, fd):
        self._fd = fd
        self._handle = msvcrt.get_osfhandle(fd)

    @property
    def fd(self):
        """The Python file descriptor."""
        return self._fd

    @property
    def handle(self):
        """The Windows console handle."""
        return self._handle

    def wait(self, timeout=None):
        """Wait for console input to become available.

        :arg float timeout: Seconds to wait.  ``None`` blocks indefinitely,
            ``0`` is non-blocking, positive values specify a deadline.
        :returns: ``True`` if input is available, ``False`` on timeout.
        :rtype: bool
        :raises OSError: If WaitForSingleObject_ fails.

        The call is non-destructive -- no events are consumed.

        .. _WaitForSingleObject:
            https://learn.microsoft.com/en-us/windows/win32/api/synchapi/nf-synchapi-waitforsingleobject
        """
        if timeout is None:
            wait_ms = 0xFFFFFFFF  # INFINITE
        else:
            wait_ms = int(1000 * max(0, timeout))

        result = KERNEL32.WaitForSingleObject(self._handle, wait_ms)

        if result == 0xFFFFFFFF:  # WAIT_FAILED
            raise ctypes.WinError(ctypes.get_last_error())

        return result == 0x00000000  # WAIT_OBJECT_0

    def peek(self):
        """Peek at the next console input event without consuming it.

        :returns: The next :py:class:`INPUT_RECORD`, or ``None`` if the
            buffer is empty.
        :rtype: INPUT_RECORD or None
        :raises OSError: If PeekConsoleInputW_ fails.

        .. _PeekConsoleInputW:
            https://learn.microsoft.com/en-us/windows/console/peekconsoleinput
        """
        record = INPUT_RECORD()
        count = wintypes.DWORD()

        if not KERNEL32.PeekConsoleInputW(
                self._handle, ctypes.byref(record), 1, ctypes.byref(count)):
            raise ctypes.WinError(ctypes.get_last_error())

        if count.value == 0:
            return None
        return record

    def read(self):
        """Read and consume the next console input event.

        :returns: The next :py:class:`INPUT_RECORD`, or ``None`` if the
            buffer is empty.
        :rtype: INPUT_RECORD or None
        :raises OSError: If ReadConsoleInputW_ fails.

        .. _ReadConsoleInputW:
            https://learn.microsoft.com/en-us/windows/console/readconsoleinput
        """
        record = INPUT_RECORD()
        count = wintypes.DWORD()

        if not KERNEL32.ReadConsoleInputW(
                self._handle, ctypes.byref(record), 1, ctypes.byref(count)):
            raise ctypes.WinError(ctypes.get_last_error())

        if count.value == 0:
            return None
        return record



"""
ansi-bbs

Source: https://gitlab.synchro.net/main/sbbs/-/raw/master/install/termcap
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

BOOL_CAPS = [
    'am',      # (auto_right_margin) terminal has automatic margins
    'eo',      # (erase_overstrike) can erase overstrikes with a blank
    'msgr',    # (move_standout_mode) safe to move while in standout mode
]

NUM_CAPS = {
    'colors': 8,    # (max_colors) maximum number of colors on screen
    'cols': 80,     # (columns) number of columns in a line
    'lines': 24,    # (lines) number of lines on screen or page
    'pairs': 64,    # (max_pairs) maximum number of color-pairs on the screen
}

STR_CAPS = {
    'acsc': b'\x7d\x9c|\x80{\x82+\x10,\x11l\x8am\x80k\xb7j\x99u\x94t\x83v\x81w\x82q\x84x\x93n\x85`^Da\xb0f\xf8g\xf1~\xf9.^Y-^Xh\xb1i^U0\x9by\xf3z\xf2',
    'enacs': b'',  # empty (G0/SO/SI stripped)
    'rmacs': b'',  # empty (G0/SO/SI stripped)
    's0ds': b'',  # empty (G0/SO/SI stripped)
    's1ds': b'',  # empty (G0/SO/SI stripped)
    'smacs': b'',  # empty (G0/SO/SI stripped)
    'bel': b'\a',
    'blink': b'\x1b[5m',
    'bold': b'\x1b[1m',
    'clear': b'\x1b[2J\x1b[H',
    'cnorm': b'\x1b[?25h',
    'civis': b'\x1b[?25l',
    'cr': b'\r',
    'cub': b'\x1b[%p1%dD',
    'cub1': b'\b',
    'cud': b'\x1b[%p1%dB',
    'cud1': b'\x1b[B',
    'cuf': b'\x1b[%p1%dC',
    'cuf1': b'\x1b[C',
    'cup': b'\x1b[%i%p1%d;%p2%dH',
    'cuu': b'\x1b[%p1%dA',
    'cuu1': b'\x1b[A',
    'cvvis': b'\x1b[?25h',
    'dch1': b'\x1b[P',
    'dl1': b'\x1b[M',
    'ed': b'\x1b[J',
    'el': b'\x1b[K',
    'el1': b'\x1b[1K',
    'home': b'\x1b[H',
    'ht': b'\t',
    'il1': b'\x1b[L',
    'ich1': b'\x1b[@',
    'invis': b'\x1b[8m',
    'is1': b'\x1b[m\x1b[2J\x1b[H',
    'kbs': b'\x08',
    'kcub1': b'\x1b[D',
    'kcud1': b'\x1b[B',
    'kcuf1': b'\x1b[C',
    'kcuu1': b'\x1b[A',
    'kdch1': b'\x7f',
    'kel': b'\x1b[K',
    'kend': b'\x1b[Y',
    'kf1': b'\x1bOP',
    'kf2': b'\x1bOQ',
    'kf3': b'\x1bOR',
    'kf4': b'\x1bOS',
    'kf5': b'\x1bOt',
    'kf6': b'\x1b[17~',
    'kf7': b'\x1b[18~',
    'kf8': b'\x1b[19~',
    'kf9': b'\x1b[20~',
    'kf10': b'\x1b[21~',
    'kf11': b'\x1b[23~',
    'kf12': b'\x1b[24~',
    'khome': b'\x1b[H',
    'kich1': b'\x1b[@',
    'knp': b'\x1b[U',
    'kpp': b'\x1b[V',
    'op': b'\x1b[m',
    'rc': b'\x1b[u',
    'rev': b'\x1b[7m',
    'rmam': b'\x1b[?7l',
    'rmcup': b'\x1b[?69h\x1b[s\x1b[?69l',
    'rs1': b'\x1b[m\x1b[2J\x1b[H',
    'sc': b'\x1b[s',
    'setab': b'\x1b[4%p1%dm',
    'setaf': b'\x1b[3%p1%dm',
    'sgr0': b'\x1b[m',
    'smam': b'\x1b[?7h',
    'smcup': b'\x1b[?69h\x1b[s\x1b[?69l',
    'smul': b'\x1b[4m',
    'u6': b'\x1b[%i%p1%d;%p2%dR',
    'u7': b'\x1b[6n',
    'vpa': b'\x1b[%i%p1%dd',
}



"""
ansi terminal info

Derived from Revision: 1.1198
Source: https://invisible-mirror.net/archives/ncurses/current/terminfo.src.gz

jquast: added capabilities commonly available in 1980s-1990s terminal emulator programs (like
minicom, telix, etc), but absent from the ANSI X3.64 standard: sc (save_cursor), rc
(restore_cursor), civis (cursor_invisible), cnorm (cursor_normal).

This file is derived from the ncurses terminfo database, which is distributed under the MIT/X11
license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

BOOL_CAPS = [
    'am',      # (auto_right_margin) terminal has automatic margins
    'mc5i',    # (prtr_silent) printer will not echo on screen
    'mir',     # (move_insert_mode) safe to move while in insert mode
    'msgr',    # (move_standout_mode) safe to move while in standout mode
]

NUM_CAPS = {
    'colors': 8,    # (max_colors) maximum number of colors on screen
    'cols': 80,     # (columns) number of columns in a line
    'it': 8,        # (init_tabs) tabs initially every # spaces
    'lines': 24,    # (lines) number of lines on screen or page
    'ncv': 3,       # (no_color_video) video attributes that cannot be used with colors
    'pairs': 64,    # (max_pairs) maximum number of color-pairs on the screen
}

STR_CAPS = {
    'acsc': b'+\x10,\x11-\x18.\x190\xdb`\x04a\xb1f\xf8g\xf1h\xb0j\xd9k\xbfl\xdam\xc0n\xc5o~p\xc4q\xc4r\xc4s_t\xc3u\xb4v\xc1w\xc2x\xb3y\xf3z\xf2{\xe3|\xd8}\x9c~\xfe',
    'bel': b'\a',
    'blink': b'\x1b[5m',
    'bold': b'\x1b[1m',
    'cbt': b'\x1b[Z',
    'clear': b'\x1b[H\x1b[J',
    'civis': b'\x1b[?25l',
    'cnorm': b'\x1b[?25h',
    'cr': b'\r',
    'cub': b'\x1b[%p1%dD',
    'cub1': b'\x1b[D',
    'cud': b'\x1b[%p1%dB',
    'cud1': b'\x1b[B',
    'cuf': b'\x1b[%p1%dC',
    'cuf1': b'\x1b[C',
    'cup': b'\x1b[%i%p1%d;%p2%dH',
    'cuu': b'\x1b[%p1%dA',
    'cuu1': b'\x1b[A',
    'dch': b'\x1b[%p1%dP',
    'dch1': b'\x1b[P',
    'dl': b'\x1b[%p1%dM',
    'dl1': b'\x1b[M',
    'ech': b'\x1b[%p1%dX',
    'ed': b'\x1b[J',
    'el': b'\x1b[K',
    'el1': b'\x1b[1K',
    'enacs': b'',
    'home': b'\x1b[H',
    'hpa': b'\x1b[%i%p1%dG',
    'ht': b'\x1b[I',
    'hts': b'\x1bH',
    'ich': b'\x1b[%p1%d@',
    'il': b'\x1b[%p1%dL',
    'il1': b'\x1b[L',
    'ind': b'\n',
    'indn': b'\x1b[%p1%dS',
    'invis': b'\x1b[8m',
    'kbs': b'\b',
    'kcbt': b'\x1b[Z',
    'kcub1': b'\x1b[D',
    'kcud1': b'\x1b[B',
    'kcuf1': b'\x1b[C',
    'kcuu1': b'\x1b[A',
    'khome': b'\x1b[H',
    'kich1': b'\x1b[L',
    'mc4': b'\x1b[4i',
    'mc5': b'\x1b[5i',
    'nel': b'\r\x1b[S',
    'op': b'\x1b[39;49m',
    'rep': b'%p1%c\x1b[%p2%{1}%-%db',
    'rc': b'\x1b[u',
    'rev': b'\x1b[7m',
    'rin': b'\x1b[%p1%dT',
    'rmacs': b'',
    'rmpch': b'\x1b[10m',
    'rmso': b'\x1b[m',
    'rmul': b'\x1b[m',
    's0ds': b'',
    's1ds': b'',
    's2ds': b'\x1b*B',
    's3ds': b'\x1b+B',
    'setab': b'\x1b[4%p1%dm',
    'setaf': b'\x1b[3%p1%dm',
    'sgr': b'\x1b[0;10%?%p1%t;7%;%?%p2%t;4%;%?%p3%t;7%;%?%p4%t;5%;%?%p6%t;1%;%?%p7%t;8%;%?%p9%t;11%;m',
    'sgr0': b'\x1b[0;10m',
    'sc': b'\x1b[s',
    'smacs': b'',
    'smpch': b'\x1b[11m',
    'smso': b'\x1b[7m',
    'smul': b'\x1b[4m',
    'tbc': b'\x1b[3g',
    'u6': b'\x1b[%i%d;%dR',
    'u7': b'\x1b[6n',
    'u8': b'\x1b[?%[;0123456789]c',
    'u9': b'\x1b[c',
    'vpa': b'\x1b[%i%p1%dd',
}



"""
linux-16color terminal info (derived from linux)

Revision: 1.1256
Source: https://invisible-mirror.net/archives/ncurses/current/ncurses.tar.gz

This file is derived from the ncurses terminfo database, which is
distributed under the MIT/X11 license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

from .linux import BOOL_CAPS, NUM_CAPS, STR_CAPS

BOOL_CAPS = BOOL_CAPS[:]
NUM_CAPS = NUM_CAPS.copy()
STR_CAPS = STR_CAPS.copy()
NUM_CAPS['colors'] = 16
NUM_CAPS['ncv'] = 42
NUM_CAPS['pairs'] = 256

# Modified strings
STR_CAPS['setab'] = b'\x1b[4%p1%{8}%m%d%?%p1%{7}%>%t;5%e;25%;m'
STR_CAPS['setaf'] = b'\x1b[3%p1%{8}%m%d%?%p1%{7}%>%t;1%e;22%;m'



"""
rxvt-256color terminal info (derived from rxvt)

Revision: 1.1256
Source: https://invisible-mirror.net/archives/ncurses/current/ncurses.tar.gz

This file is derived from the ncurses terminfo database, which is
distributed under the MIT/X11 license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

from .rxvt import BOOL_CAPS, NUM_CAPS, STR_CAPS

BOOL_CAPS = BOOL_CAPS[:]
NUM_CAPS = NUM_CAPS.copy()
STR_CAPS = STR_CAPS.copy()
BOOL_CAPS.append('ccc')
NUM_CAPS['colors'] = 256
NUM_CAPS['pairs'] = 65536

# Added strings
STR_CAPS['initc'] = b'\x1b]4;%p1%d;rgb:%p2%{255}%*%{1000}%/%2.2X/%p3%{255}%*%{1000}%/%2.2X/%p4%{255}%*%{1000}%/%2.2X\x1b\x5c'
STR_CAPS['oc'] = b'\x1b]104\a'

# Modified strings
STR_CAPS['setab'] = b'\x1b[%?%p1%{8}%<%t4%p1%d%e%p1%{16}%<%t10%p1%{8}%-%d%e48;5;%p1%d%;m'
STR_CAPS['setaf'] = b'\x1b[%?%p1%{8}%<%t3%p1%d%e%p1%{16}%<%t9%p1%{8}%-%d%e38;5;%p1%d%;m'



"""
jinxed terminal info library

Most of this information came from the terminfo man pages, part of ncurses
More information on ncurses can be found at:
https://www.gnu.org/software/ncurses/ncurses.html

Boolean and numeric capabilities are listed here to support tigetnum() and tigetflag()
"""

# pylint: disable=wrong-spelling-in-comment

BOOL_CAPS = [
    'AX',  # (ansi_x3.64_1979) terminal uses ECMA-48/ANSI X3.64 color sequences
    'am',  # (auto_right_margin) terminal has automatic margins
    'bce',  # (back_color_erase) screen erased with background color
    'bw',  # (auto_left_margin) cub1 wraps from column 0 to last column
    'ccc',  # (can_change) terminal can re-define existing colors
    'chts',  # (hard_cursor) cursor is hard to see
    'cpix',  # (cpi_changes_res) changing character pitch changes resolution
    'crxm',  # (cr_cancels_micro_mode) using cr turns off micro mode
    'daisy',  # (has_print_wheel) printer needs operator to change character set
    'da',  # (memory_above) display may be retained above the screen
    'db',  # (memory_below) display may be retained below the screen
    'eo',  # (erase_overstrike) can erase overstrikes with a blank
    'eslok',  # (status_line_esc_ok) escape can be used on the status line
    'gn',  # (generic_type) generic line type
    'hc',  # (hard_copy) hardcopy terminal
    'hls',  # (hue_lightness_saturation) terminal uses only HLS color notation (Tektronix)
    'hs',  # (has_status_line) has extra status line
    'hz',  # (tilde_glitch) cannot print ~'s (Hazeltine)
    'in',  # (insert_null_glitch) insert mode distinguishes nulls
    'km',  # (has_meta_key) Has a meta key (i.e., sets 8th-bit)
    'lpix',  # (lpi_changes_res) changing line pitch changes resolution
    'mc5i',  # (prtr_silent) printer will not echo on screen
    'mir',  # (move_insert_mode) safe to move while in insert mode
    'msgr',  # (move_standout_mode) safe to move while in standout mode
    'ndscr',  # (non_dest_scroll_region) scrolling region is non-destructive
    'npc',  # (no_pad_char) pad character does not exist
    'nrrmc',  # (non_rev_rmcup) smcup does not reverse rmcup
    'nxon',  # (needs_xon_xoff) padding will not work, xon/xoff required
    'os',  # (over_strike) terminal can overstrike
    'sam',  # (semi_auto_right_margin) printing in last column causes cr
    'ul',  # (transparent_underline) underline character overstrikes
    'xenl',  # (eat_newline_glitch) newline ignored after 80 cols (concept)
    'xhpa',  # (col_addr_glitch) only positive motion for hpa/mhpa caps
    'xhp',  # (ceol_standout_glitch) standout not erased by overwriting (hp)
    'xon',  # (xon_xoff) terminal uses xon/xoff handshaking
    'xsb',  # (no_esc_ctlc) beehive (f1=escape, f2=ctrl C)
    'xt',  # (dest_tabs_magic_smso) tabs destructive, magic so char (t1061)
    'xvpa',  # (row_addr_glitch) only positive motion for vpa/mvpa caps
]

NUM_CAPS = [
    'bitwin',  # (bit_image_entwining) number of passes for each bit-image row
    'bitype',  # (bit_image_type) type of bit-image device
    'btns',  # (buttons) number of buttons on mouse
    'bufsz',  # (buffer_capacity) numbers of bytes buffered before printing
    'colors',  # (max_colors) maximum number of colors on screen
    'cols',  # (columns) number of columns in a line
    'cps',  # (print_rate) print rate in characters per second
    'it',  # (init_tabs) tabs initially every # spaces
    'lh',  # (label_height) rows in each label
    'lines',  # (lines) number of lines on screen or page
    'lm',  # (lines_of_memory) lines of memory if > line. 0 means varies
    'lw',  # (label_width) columns in each label
    'ma',  # (max_attributes) maximum combined attributes terminal can handle
    'maddr',  # (max_micro_address) maximum value in micro_..._address
    'mcs',  # (micro_col_size) character step size when in micro mode
    'mjump',  # (max_micro_jump) maximum value in parm_..._micro
    'mls',  # (micro_line_size) line step size when in micro mode
    'ncv',  # (no_color_video) video attributes that cannot be used with colors
    'nlab',  # (num_labels) number of labels on screen
    'npins',  # (number_of_pins) numbers of pins in print-head
    'orc',  # (output_res_char) horizontal resolution in units per line
    'orhi',  # (output_res_horz_inch) horizontal resolution in units per inch
    'orl',  # (output_res_line) vertical resolution in units per line
    'orvi',  # (output_res_vert_inch) vertical resolution in units per inch
    'pairs',  # (max_pairs) maximum number of color-pairs on the screen
    'pb',  # (padding_baud_rate) lowest baud rate where padding needed
    'spinh',  # (dot_horz_spacing) spacing of dots horizontally in dots per inch
    'spinv',  # (dot_vert_spacing) spacing of pins vertically in pins per inch
    'U8',  # (utf8_terminal) ncurses uses Unicode values for line-drawing in UTF-8 locale
    'vt',  # (virtual_terminal) virtual terminal number (CB/unix)
    'widcs',  # (wide_char_size) character step size when in double wide mode
    'wnum',  # (maximum_windows) maximum number of definable windows
    'wsl',  # (width_status_line) number of columns in status line
    'xmc',  # (magic_cookie_glitch) number of blank characters left by smso or rmso
]



"""Auto-generated terminal name aliases."""
# Auto-generated by codegen_terminfo.py
# Maps alternative terminal names to the primary ncurses name.
# Primary names are module names (with '-' and '.' replaced by '_').

ALIASES = {
    'X-hpterm': 'hpterm',
    'ansi80x25': 'cons25',
    'ansis': 'cons25',
    'contour-latest': 'contour',
    'iris-ansi-net': 'iris-ansi',
    'iterm2': 'iTerm2.app',
    'rxvt-color': 'rxvt',
    'stterm': 'st',
    'stterm-256color': 'st-256color',
    'sun1': 'sun',
    'sun2': 'sun',
    'vt100-am': 'vt100',
    'vt200': 'vt220',
    'vt300': 'vt320',
    'wyse50': 'wy50',
    'xterm-ghostty': 'ghostty',
    'xterm-kitty': 'kitty',
}



"""
cons25 terminal info

Revision: 1.1256
Source: https://invisible-mirror.net/archives/ncurses/current/ncurses.tar.gz

This file is derived from the ncurses terminfo database, which is
distributed under the MIT/X11 license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

BOOL_CAPS = [
    'am',      # (auto_right_margin) terminal has automatic margins
    'bce',     # (back_color_erase) screen erased with background color
    'bw',      # (auto_left_margin) cub1 wraps from column 0 to last column
    'eo',      # (erase_overstrike) can erase overstrikes with a blank
    'msgr',    # (move_standout_mode) safe to move while in standout mode
    'npc',     # (no_pad_char) pad character does not exist
]

NUM_CAPS = {
    'colors': 8,    # (max_colors) maximum number of colors on screen
    'cols': 80,     # (columns) number of columns in a line
    'it': 8,        # (init_tabs) tabs initially every # spaces
    'lines': 25,    # (lines) number of lines on screen or page
    'ncv': 21,      # (no_color_video) video attributes that cannot be used with colors
    'pairs': 64,    # (max_pairs) maximum number of color-pairs on the screen
}

STR_CAPS = {
    'acsc': b'-\x18.\x190\xdb`\x04a\xb0f\xf8g\xf1h\xb1i\x15j\xd9k\xbfl\xdam\xc0n\xc5q\xc4t\xc3u\xb4v\xc1w\xc2x\xb3y\xf3z\xf2~\xf9',
    'bel': b'\a',
    'blink': b'\x1b[5m',
    'bold': b'\x1b[1m',
    'cbt': b'\x1b[Z',
    'clear': b'\x1b[H\x1b[J',
    'cnorm': b'\x1b[=0C',
    'cr': b'\r',
    'cub': b'\x1b[%p1%dD',
    'cub1': b'\b',
    'cud': b'\x1b[%p1%dB',
    'cud1': b'\x1b[B',
    'cuf': b'\x1b[%p1%dC',
    'cuf1': b'\x1b[C',
    'cup': b'\x1b[%i%p1%d;%p2%dH',
    'cuu': b'\x1b[%p1%dA',
    'cuu1': b'\x1b[A',
    'cvvis': b'\x1b[=1C',
    'dch': b'\x1b[%p1%dP',
    'dch1': b'\x1b[P',
    'dim': b'\x1b[30;1m',
    'dl': b'\x1b[%p1%dM',
    'dl1': b'\x1b[M',
    'ech': b'\x1b[%p1%dX',
    'ed': b'\x1b[J',
    'el': b'\x1b[K',
    'enacs': b'',
    'home': b'\x1b[H',
    'hpa': b'\x1b[%i%p1%d`',
    'ht': b'\t',
    'ich': b'\x1b[%p1%d@',
    'ich1': b'\x1b[@',
    'il': b'\x1b[%p1%dL',
    'il1': b'\x1b[L',
    'ind': b'\x1b[S',
    'indn': b'\x1b[%p1%dS',
    'kb2': b'\x1b[E',
    'kbs': b'\b',
    'kcbt': b'\x1b[Z',
    'kcub1': b'\x1b[D',
    'kcud1': b'\x1b[B',
    'kcuf1': b'\x1b[C',
    'kcuu1': b'\x1b[A',
    'kdch1': b'\x7f',
    'kend': b'\x1b[F',
    'kf1': b'\x1b[M',
    'kf10': b'\x1b[V',
    'kf11': b'\x1b[W',
    'kf12': b'\x1b[X',
    'kf13': b'\x1b[Y',
    'kf14': b'\x1b[Z',
    'kf15': b'\x1b[a',
    'kf16': b'\x1b[b',
    'kf17': b'\x1b[c',
    'kf18': b'\x1b[d',
    'kf19': b'\x1b[e',
    'kf2': b'\x1b[N',
    'kf20': b'\x1b[f',
    'kf21': b'\x1b[g',
    'kf22': b'\x1b[h',
    'kf23': b'\x1b[i',
    'kf24': b'\x1b[j',
    'kf25': b'\x1b[k',
    'kf26': b'\x1b[l',
    'kf27': b'\x1b[m',
    'kf28': b'\x1b[n',
    'kf29': b'\x1b[o',
    'kf3': b'\x1b[O',
    'kf30': b'\x1b[p',
    'kf31': b'\x1b[q',
    'kf32': b'\x1b[r',
    'kf33': b'\x1b[s',
    'kf34': b'\x1b[t',
    'kf35': b'\x1b[u',
    'kf36': b'\x1b[v',
    'kf37': b'\x1b[w',
    'kf38': b'\x1b[x',
    'kf39': b'\x1b[y',
    'kf4': b'\x1b[P',
    'kf40': b'\x1b[z',
    'kf41': b'\x1b[@',
    'kf42': b'\x1b[[',
    'kf43': b'\x1b[\x5c',
    'kf44': b'\x1b[]',
    'kf45': b'\x1b[^',
    'kf46': b'\x1b[_',
    'kf47': b'\x1b[`',
    'kf48': b'\x1b[{',
    'kf5': b'\x1b[Q',
    'kf6': b'\x1b[R',
    'kf7': b'\x1b[S',
    'kf8': b'\x1b[T',
    'kf9': b'\x1b[U',
    'khome': b'\x1b[H',
    'kich1': b'\x1b[L',
    'knp': b'\x1b[G',
    'kpp': b'\x1b[I',
    'nel': b'\x1b[E',
    'op': b'\x1b[x',
    'rc': b'\x1b8',
    'rev': b'\x1b[7m',
    'ri': b'\x1b[T',
    'rin': b'\x1b[%p1%dT',
    'rmacs': b'',
    'rmso': b'\x1b[m',
    'rs2': b'\x1b[x\x1b[m\x1bc',
    's0ds': b'',
    's1ds': b'',
    'sc': b'\x1b7',
    'setab': b'\x1b[4%p1%dm',
    'setaf': b'\x1b[3%p1%dm',
    'sgr': b'\x1b[0%?%p1%t;2;7%;%?%p3%t;7%;%?%p4%t;5%;%?%p5%t;30;1%;%?%p6%t;1%;m',
    'sgr0': b'\x1b[m',
    'smacs': b'',
    'smso': b'\x1b[7m',
    'vpa': b'\x1b[%i%p1%dd',
}



"""
wy50 terminal info

Revision: 1.1256
Source: https://invisible-mirror.net/archives/ncurses/current/ncurses.tar.gz

This file is derived from the ncurses terminfo database, which is
distributed under the MIT/X11 license.  See LICENSE.ncurses.
"""

# flake8: noqa: E501
# pylint: disable=line-too-long

BOOL_CAPS = [
    'am',      # (auto_right_margin) terminal has automatic margins
    'bw',      # (auto_left_margin) cub1 wraps from column 0 to last column
    'hs',      # (has_status_line) has extra status line
    'mc5i',    # (prtr_silent) printer will not echo on screen
    'mir',     # (move_insert_mode) safe to move while in insert mode
    'msgr',    # (move_standout_mode) safe to move while in standout mode
    'xon',     # (xon_xoff) terminal uses xon/xoff handshaking
]

NUM_CAPS = {
    'cols': 80,     # (columns) number of columns in a line
    'lh': 1,        # (label_height) rows in each label
    'lines': 24,    # (lines) number of lines on screen or page
    'lw': 8,        # (label_width) columns in each label
    'ma': 1,        # (max_attributes) maximum combined attributes terminal can handle
    'nlab': 8,      # (num_labels) number of labels on screen
    'wsl': 45,      # (width_status_line) number of columns in status line
}

STR_CAPS = {
    'acsc': b'a;j5k3l2m1n8q:t4u9v=w0x6',
    'bel': b'\a',
    'cbt': b'\x1bI',
    'civis': b'\x1b`0',
    'clear': b'\x1b+',
    'cnorm': b'\x1b`1',
    'cr': b'\r',
    'cub1': b'\b',
    'cud1': b'\n',
    'cuf1': b'\f',
    'cup': b'\x1b=%p1%\x27 \x27%+%c%p2%\x27 \x27%+%c',
    'cuu1': b'\v',
    'dch1': b'\x1bW',
    'dim': b'\x1b`7\x1b)',
    'dl1': b'\x1bR',
    'dsl': b'\x1bF\r',
    'ed': b'\x1bY',
    'el': b'\x1bT',
    'enacs': b'',
    'flash': b'\x1b`8\x1b`9',
    'fsl': b'\r',
    'home': b'\x1e',
    'ht': b'\t',
    'hts': b'\x1b1',
    'il1': b'\x1bE',
    'ind': b'\n',
    'is1': b'\x1b`:\x1b`9',
    'is2': b'\x14\x1b\x27\x1b(',
    'kF1': b'\x01`\r',
    'kF10': b'\x01i\r',
    'kF11': b'\x01j\r',
    'kF12': b'\x01k\r',
    'kF13': b'\x01l\r',
    'kF14': b'\x01m\r',
    'kF15': b'\x01n\r',
    'kF16': b'\x01o\r',
    'kF2': b'\x01a\r',
    'kF3': b'\x01b\r',
    'kF4': b'\x01c\r',
    'kF5': b'\x01d\r',
    'kF6': b'\x01e\r',
    'kF7': b'\x01f\r',
    'kF8': b'\x01g\r',
    'kF9': b'\x01h\r',
    'kHOM': b'\x1b{',
    'kbs': b'\b',
    'kcbt': b'\x1bI',
    'kcub1': b'\b',
    'kcud1': b'\n',
    'kcuf1': b'\f',
    'kcuu1': b'\v',
    'kdch1': b'\x1bW',
    'kdl1': b'\x1bR',
    'ked': b'\x1bY',
    'kel': b'\x1bT',
    'kent': b'\x1b7',
    'kf1': b'\x01@\r',
    'kf10': b'\x01I\r',
    'kf11': b'\x01J\r',
    'kf12': b'\x01K\r',
    'kf13': b'\x01L\r',
    'kf14': b'\x01M\r',
    'kf15': b'\x01N\r',
    'kf16': b'\x01O\r',
    'kf2': b'\x01A\r',
    'kf3': b'\x01B\r',
    'kf4': b'\x01C\r',
    'kf5': b'\x01D\r',
    'kf6': b'\x01E\r',
    'kf7': b'\x01F\r',
    'kf8': b'\x01G\r',
    'kf9': b'\x01H\r',
    'khome': b'\x1e',
    'kich1': b'\x1bQ',
    'kil1': b'\x1bE',
    'knp': b'\x1bK',
    'kpp': b'\x1bJ',
    'kprt': b'\x1bP',
    'krpl': b'\x1br',
    'll': b'\x1e\v',
    'mc0': b'\x1bP',
    'mc4': b'\x14',
    'mc5': b'\x18',
    'nel': b'\r\n',
    'pfx': b'\x1bz%p1%\x27?\x27%+%c%p2%s\x7f',
    'pln': b'\x1bz%p1%\x27/\x27%+%c%p2%s\r',
    'prot': b'\x1b`7\x1b)',
    'rev': b'\x1b`6\x1b)',
    'ri': b'\x1bj',
    'rmacs': b'',
    'rmir': b'\x1br',
    'rmln': b'\x1bA11',
    'rmso': b'\x1b(',
    's0ds': b'',
    's1ds': b'',
    'sgr': b'%?%p1%p3%|%t\x1b`6\x1b)%e%p5%p8%|%t\x1b`7\x1b)%e\x1b(%;%?%p9%t\x1bH\x02%e\x1bH\x03%;',
    'sgr0': b'\x1b(\x1bH\x03',
    'smacs': b'',
    'smir': b'\x1bq',
    'smln': b'\x1bA10',
    'smso': b'\x1b`6\x1b)',
    'tbc': b'\x1b0',
    'tsl': b'\x1bF',
}
